// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>

#include "base/check_op.h"
#include "third_party/blink/public/mojom/use_counter/metrics/css_property_id.mojom-blink-forward.h"
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator/allocator.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/hash_traits.h"

namespace WTF {
class AtomicString;
class String;
}  // namespace WTF

namespace blink {

class ExecutionContext;

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kColor = 2,
    kDirection = 3,
    kFontFamily = 4,
    kFontFeatureSettings = 5,
    kFontKerning = 6,
    kFontOpticalSizing = 7,
    kFontPalette = 8,
    kFontSize = 9,
    kFontSizeAdjust = 10,
    kFontStretch = 11,
    kFontStyle = 12,
    kFontSynthesisSmallCaps = 13,
    kFontSynthesisStyle = 14,
    kFontSynthesisWeight = 15,
    kFontVariantAlternates = 16,
    kFontVariantCaps = 17,
    kFontVariantEastAsian = 18,
    kFontVariantLigatures = 19,
    kFontVariantNumeric = 20,
    kFontVariantPosition = 21,
    kFontVariationSettings = 22,
    kFontWeight = 23,
    kForcedColorAdjust = 24,
    kInternalVisitedColor = 25,
    kTextOrientation = 26,
    kTextRendering = 27,
    kWebkitFontSmoothing = 28,
    kWebkitLocale = 29,
    kWebkitTextOrientation = 30,
    kWebkitWritingMode = 31,
    kWritingMode = 32,
    kZoom = 33,
    kAccentColor = 34,
    kAdditiveSymbols = 35,
    kAlignContent = 36,
    kAlignItems = 37,
    kAlignSelf = 38,
    kAlignmentBaseline = 39,
    kAll = 40,
    kAnchorDefault = 41,
    kAnchorName = 42,
    kAnimationComposition = 43,
    kAnimationDelay = 44,
    kAnimationDelayEnd = 45,
    kAnimationDelayStart = 46,
    kAnimationDirection = 47,
    kAnimationDuration = 48,
    kAnimationFillMode = 49,
    kAnimationIterationCount = 50,
    kAnimationName = 51,
    kAnimationPlayState = 52,
    kAnimationRangeEnd = 53,
    kAnimationRangeStart = 54,
    kAnimationTimeline = 55,
    kAnimationTimingFunction = 56,
    kAppRegion = 57,
    kAppearance = 58,
    kAscentOverride = 59,
    kAspectRatio = 60,
    kBackdropFilter = 61,
    kBackfaceVisibility = 62,
    kBackgroundAttachment = 63,
    kBackgroundBlendMode = 64,
    kBackgroundClip = 65,
    kBackgroundColor = 66,
    kBackgroundImage = 67,
    kBackgroundOrigin = 68,
    kBackgroundPositionX = 69,
    kBackgroundPositionY = 70,
    kBackgroundRepeatX = 71,
    kBackgroundRepeatY = 72,
    kBackgroundSize = 73,
    kBasePalette = 74,
    kBaselineShift = 75,
    kBaselineSource = 76,
    kBlockSize = 77,
    kBorderBlockEndColor = 78,
    kBorderBlockEndStyle = 79,
    kBorderBlockEndWidth = 80,
    kBorderBlockStartColor = 81,
    kBorderBlockStartStyle = 82,
    kBorderBlockStartWidth = 83,
    kBorderBottomColor = 84,
    kBorderBottomLeftRadius = 85,
    kBorderBottomRightRadius = 86,
    kBorderBottomStyle = 87,
    kBorderBottomWidth = 88,
    kBorderCollapse = 89,
    kBorderEndEndRadius = 90,
    kBorderEndStartRadius = 91,
    kBorderImageOutset = 92,
    kBorderImageRepeat = 93,
    kBorderImageSlice = 94,
    kBorderImageSource = 95,
    kBorderImageWidth = 96,
    kBorderInlineEndColor = 97,
    kBorderInlineEndStyle = 98,
    kBorderInlineEndWidth = 99,
    kBorderInlineStartColor = 100,
    kBorderInlineStartStyle = 101,
    kBorderInlineStartWidth = 102,
    kBorderLeftColor = 103,
    kBorderLeftStyle = 104,
    kBorderLeftWidth = 105,
    kBorderRightColor = 106,
    kBorderRightStyle = 107,
    kBorderRightWidth = 108,
    kBorderStartEndRadius = 109,
    kBorderStartStartRadius = 110,
    kBorderTopColor = 111,
    kBorderTopLeftRadius = 112,
    kBorderTopRightRadius = 113,
    kBorderTopStyle = 114,
    kBorderTopWidth = 115,
    kBottom = 116,
    kBoxShadow = 117,
    kBoxSizing = 118,
    kBreakAfter = 119,
    kBreakBefore = 120,
    kBreakInside = 121,
    kBufferedRendering = 122,
    kCaptionSide = 123,
    kCaretColor = 124,
    kClear = 125,
    kClip = 126,
    kClipPath = 127,
    kClipRule = 128,
    kColorInterpolation = 129,
    kColorInterpolationFilters = 130,
    kColorRendering = 131,
    kColorScheme = 132,
    kColumnCount = 133,
    kColumnFill = 134,
    kColumnGap = 135,
    kColumnRuleColor = 136,
    kColumnRuleStyle = 137,
    kColumnRuleWidth = 138,
    kColumnSpan = 139,
    kColumnWidth = 140,
    kContain = 141,
    kContainIntrinsicBlockSize = 142,
    kContainIntrinsicHeight = 143,
    kContainIntrinsicInlineSize = 144,
    kContainIntrinsicWidth = 145,
    kContainerName = 146,
    kContainerType = 147,
    kContent = 148,
    kContentVisibility = 149,
    kCounterIncrement = 150,
    kCounterReset = 151,
    kCounterSet = 152,
    kCursor = 153,
    kCx = 154,
    kCy = 155,
    kD = 156,
    kDescentOverride = 157,
    kDisplay = 158,
    kDominantBaseline = 159,
    kDynamicRangeLimit = 160,
    kEmptyCells = 161,
    kFallback = 162,
    kFill = 163,
    kFillOpacity = 164,
    kFillRule = 165,
    kFilter = 166,
    kFlexBasis = 167,
    kFlexDirection = 168,
    kFlexGrow = 169,
    kFlexShrink = 170,
    kFlexWrap = 171,
    kFloat = 172,
    kFloodColor = 173,
    kFloodOpacity = 174,
    kFontDisplay = 175,
    kFormSizing = 176,
    kGridAutoColumns = 177,
    kGridAutoFlow = 178,
    kGridAutoRows = 179,
    kGridColumnEnd = 180,
    kGridColumnStart = 181,
    kGridRowEnd = 182,
    kGridRowStart = 183,
    kGridTemplateAreas = 184,
    kGridTemplateColumns = 185,
    kGridTemplateRows = 186,
    kHeight = 187,
    kHyphenateCharacter = 188,
    kHyphenateLimitChars = 189,
    kHyphens = 190,
    kImageOrientation = 191,
    kImageRendering = 192,
    kInherits = 193,
    kInitialLetter = 194,
    kInitialValue = 195,
    kInlineSize = 196,
    kInsetBlockEnd = 197,
    kInsetBlockStart = 198,
    kInsetInlineEnd = 199,
    kInsetInlineStart = 200,
    kInternalAlignContentBlock = 201,
    kInternalEmptyLineHeight = 202,
    kInternalFontSizeDelta = 203,
    kInternalForcedBackgroundColor = 204,
    kInternalForcedBorderColor = 205,
    kInternalForcedColor = 206,
    kInternalForcedOutlineColor = 207,
    kInternalForcedVisitedColor = 208,
    kInternalOverflowBlock = 209,
    kInternalOverflowInline = 210,
    kInternalVisitedBackgroundColor = 211,
    kInternalVisitedBorderBlockEndColor = 212,
    kInternalVisitedBorderBlockStartColor = 213,
    kInternalVisitedBorderBottomColor = 214,
    kInternalVisitedBorderInlineEndColor = 215,
    kInternalVisitedBorderInlineStartColor = 216,
    kInternalVisitedBorderLeftColor = 217,
    kInternalVisitedBorderRightColor = 218,
    kInternalVisitedBorderTopColor = 219,
    kInternalVisitedCaretColor = 220,
    kInternalVisitedColumnRuleColor = 221,
    kInternalVisitedFill = 222,
    kInternalVisitedOutlineColor = 223,
    kInternalVisitedStroke = 224,
    kInternalVisitedTextDecorationColor = 225,
    kInternalVisitedTextEmphasisColor = 226,
    kInternalVisitedTextFillColor = 227,
    kInternalVisitedTextStrokeColor = 228,
    kIsolation = 229,
    kJustifyContent = 230,
    kJustifyItems = 231,
    kJustifySelf = 232,
    kLeft = 233,
    kLetterSpacing = 234,
    kLightingColor = 235,
    kLineBreak = 236,
    kLineGapOverride = 237,
    kLineHeight = 238,
    kListStyleImage = 239,
    kListStylePosition = 240,
    kListStyleType = 241,
    kMarginBlockEnd = 242,
    kMarginBlockStart = 243,
    kMarginBottom = 244,
    kMarginInlineEnd = 245,
    kMarginInlineStart = 246,
    kMarginLeft = 247,
    kMarginRight = 248,
    kMarginTop = 249,
    kMarkerEnd = 250,
    kMarkerMid = 251,
    kMarkerStart = 252,
    kMask = 253,
    kMaskType = 254,
    kMathDepth = 255,
    kMathShift = 256,
    kMathStyle = 257,
    kMaxBlockSize = 258,
    kMaxHeight = 259,
    kMaxInlineSize = 260,
    kMaxWidth = 261,
    kMinBlockSize = 262,
    kMinHeight = 263,
    kMinInlineSize = 264,
    kMinWidth = 265,
    kMixBlendMode = 266,
    kNavigationTrigger = 267,
    kNegative = 268,
    kObjectFit = 269,
    kObjectPosition = 270,
    kObjectViewBox = 271,
    kOffsetAnchor = 272,
    kOffsetDistance = 273,
    kOffsetPath = 274,
    kOffsetPosition = 275,
    kOffsetRotate = 276,
    kOpacity = 277,
    kOrder = 278,
    kOriginTrialTestProperty = 279,
    kOrphans = 280,
    kOutlineColor = 281,
    kOutlineOffset = 282,
    kOutlineStyle = 283,
    kOutlineWidth = 284,
    kOverflowAnchor = 285,
    kOverflowBlock = 286,
    kOverflowClipMargin = 287,
    kOverflowInline = 288,
    kOverflowWrap = 289,
    kOverflowX = 290,
    kOverflowY = 291,
    kOverlay = 292,
    kOverrideColors = 293,
    kOverscrollBehaviorBlock = 294,
    kOverscrollBehaviorInline = 295,
    kOverscrollBehaviorX = 296,
    kOverscrollBehaviorY = 297,
    kPad = 298,
    kPaddingBlockEnd = 299,
    kPaddingBlockStart = 300,
    kPaddingBottom = 301,
    kPaddingInlineEnd = 302,
    kPaddingInlineStart = 303,
    kPaddingLeft = 304,
    kPaddingRight = 305,
    kPaddingTop = 306,
    kPage = 307,
    kPageOrientation = 308,
    kPaintOrder = 309,
    kPerspective = 310,
    kPerspectiveOrigin = 311,
    kPointerEvents = 312,
    kPopoverHideDelay = 313,
    kPopoverShowDelay = 314,
    kPosition = 315,
    kPositionFallback = 316,
    kPositionFallbackBounds = 317,
    kPrefix = 318,
    kQuotes = 319,
    kR = 320,
    kRange = 321,
    kResize = 322,
    kRight = 323,
    kRotate = 324,
    kRowGap = 325,
    kRubyPosition = 326,
    kRx = 327,
    kRy = 328,
    kScale = 329,
    kScrollBehavior = 330,
    kScrollCustomization = 331,
    kScrollMarginBlockEnd = 332,
    kScrollMarginBlockStart = 333,
    kScrollMarginBottom = 334,
    kScrollMarginInlineEnd = 335,
    kScrollMarginInlineStart = 336,
    kScrollMarginLeft = 337,
    kScrollMarginRight = 338,
    kScrollMarginTop = 339,
    kScrollPaddingBlockEnd = 340,
    kScrollPaddingBlockStart = 341,
    kScrollPaddingBottom = 342,
    kScrollPaddingInlineEnd = 343,
    kScrollPaddingInlineStart = 344,
    kScrollPaddingLeft = 345,
    kScrollPaddingRight = 346,
    kScrollPaddingTop = 347,
    kScrollSnapAlign = 348,
    kScrollSnapStop = 349,
    kScrollSnapType = 350,
    kScrollStartBlock = 351,
    kScrollStartInline = 352,
    kScrollStartTargetBlock = 353,
    kScrollStartTargetInline = 354,
    kScrollStartTargetX = 355,
    kScrollStartTargetY = 356,
    kScrollStartX = 357,
    kScrollStartY = 358,
    kScrollTimelineAxis = 359,
    kScrollTimelineName = 360,
    kScrollbarColor = 361,
    kScrollbarGutter = 362,
    kScrollbarWidth = 363,
    kShapeImageThreshold = 364,
    kShapeMargin = 365,
    kShapeOutside = 366,
    kShapeRendering = 367,
    kSize = 368,
    kSizeAdjust = 369,
    kSpeak = 370,
    kSpeakAs = 371,
    kSrc = 372,
    kStopColor = 373,
    kStopOpacity = 374,
    kStroke = 375,
    kStrokeDasharray = 376,
    kStrokeDashoffset = 377,
    kStrokeLinecap = 378,
    kStrokeLinejoin = 379,
    kStrokeMiterlimit = 380,
    kStrokeOpacity = 381,
    kStrokeWidth = 382,
    kSuffix = 383,
    kSymbols = 384,
    kSyntax = 385,
    kSystem = 386,
    kTabSize = 387,
    kTableLayout = 388,
    kTextAlign = 389,
    kTextAlignLast = 390,
    kTextAnchor = 391,
    kTextAutospace = 392,
    kTextBoxTrim = 393,
    kTextCombineUpright = 394,
    kTextDecorationColor = 395,
    kTextDecorationLine = 396,
    kTextDecorationSkipInk = 397,
    kTextDecorationStyle = 398,
    kTextDecorationThickness = 399,
    kTextEmphasisColor = 400,
    kTextEmphasisPosition = 401,
    kTextEmphasisStyle = 402,
    kTextIndent = 403,
    kTextOverflow = 404,
    kTextShadow = 405,
    kTextSizeAdjust = 406,
    kTextTransform = 407,
    kTextUnderlineOffset = 408,
    kTextUnderlinePosition = 409,
    kTextWrap = 410,
    kTimelineScope = 411,
    kToggleGroup = 412,
    kToggleRoot = 413,
    kToggleTrigger = 414,
    kToggleVisibility = 415,
    kTop = 416,
    kTouchAction = 417,
    kTransform = 418,
    kTransformBox = 419,
    kTransformOrigin = 420,
    kTransformStyle = 421,
    kTransitionBehavior = 422,
    kTransitionDelay = 423,
    kTransitionDuration = 424,
    kTransitionProperty = 425,
    kTransitionTimingFunction = 426,
    kTranslate = 427,
    kUnicodeBidi = 428,
    kUnicodeRange = 429,
    kUserSelect = 430,
    kVectorEffect = 431,
    kVerticalAlign = 432,
    kViewTimelineAxis = 433,
    kViewTimelineInset = 434,
    kViewTimelineName = 435,
    kViewTransitionName = 436,
    kVisibility = 437,
    kWebkitBorderHorizontalSpacing = 438,
    kWebkitBorderImage = 439,
    kWebkitBorderVerticalSpacing = 440,
    kWebkitBoxAlign = 441,
    kWebkitBoxDecorationBreak = 442,
    kWebkitBoxDirection = 443,
    kWebkitBoxFlex = 444,
    kWebkitBoxOrdinalGroup = 445,
    kWebkitBoxOrient = 446,
    kWebkitBoxPack = 447,
    kWebkitBoxReflect = 448,
    kWebkitLineBreak = 449,
    kWebkitLineClamp = 450,
    kWebkitMaskBoxImageOutset = 451,
    kWebkitMaskBoxImageRepeat = 452,
    kWebkitMaskBoxImageSlice = 453,
    kWebkitMaskBoxImageSource = 454,
    kWebkitMaskBoxImageWidth = 455,
    kWebkitMaskClip = 456,
    kWebkitMaskComposite = 457,
    kWebkitMaskImage = 458,
    kWebkitMaskOrigin = 459,
    kWebkitMaskPositionX = 460,
    kWebkitMaskPositionY = 461,
    kWebkitMaskRepeatX = 462,
    kWebkitMaskRepeatY = 463,
    kWebkitMaskSize = 464,
    kWebkitPerspectiveOriginX = 465,
    kWebkitPerspectiveOriginY = 466,
    kWebkitPrintColorAdjust = 467,
    kWebkitRtlOrdering = 468,
    kWebkitRubyPosition = 469,
    kWebkitTapHighlightColor = 470,
    kWebkitTextCombine = 471,
    kWebkitTextDecorationsInEffect = 472,
    kWebkitTextFillColor = 473,
    kWebkitTextSecurity = 474,
    kWebkitTextStrokeColor = 475,
    kWebkitTextStrokeWidth = 476,
    kWebkitTransformOriginX = 477,
    kWebkitTransformOriginY = 478,
    kWebkitTransformOriginZ = 479,
    kWebkitUserDrag = 480,
    kWebkitUserModify = 481,
    kWhiteSpaceCollapse = 482,
    kWidows = 483,
    kWidth = 484,
    kWillChange = 485,
    kWordBreak = 486,
    kWordSpacing = 487,
    kX = 488,
    kY = 489,
    kZIndex = 490,
    kAlternativeAnimationDelay = 491,
    kAlternativeAnimationWithDelayStartEnd = 492,
    kAlternativeAnimationWithTimeline = 493,
    kAlternativeViewTimelineWithInset = 494,
    kAnimation = 495,
    kAnimationRange = 496,
    kBackground = 497,
    kBackgroundPosition = 498,
    kBackgroundRepeat = 499,
    kBorder = 500,
    kBorderBlock = 501,
    kBorderBlockColor = 502,
    kBorderBlockEnd = 503,
    kBorderBlockStart = 504,
    kBorderBlockStyle = 505,
    kBorderBlockWidth = 506,
    kBorderBottom = 507,
    kBorderColor = 508,
    kBorderImage = 509,
    kBorderInline = 510,
    kBorderInlineColor = 511,
    kBorderInlineEnd = 512,
    kBorderInlineStart = 513,
    kBorderInlineStyle = 514,
    kBorderInlineWidth = 515,
    kBorderLeft = 516,
    kBorderRadius = 517,
    kBorderRight = 518,
    kBorderSpacing = 519,
    kBorderStyle = 520,
    kBorderTop = 521,
    kBorderWidth = 522,
    kColumnRule = 523,
    kColumns = 524,
    kContainIntrinsicSize = 525,
    kContainer = 526,
    kFlex = 527,
    kFlexFlow = 528,
    kFont = 529,
    kFontSynthesis = 530,
    kFontVariant = 531,
    kGap = 532,
    kGrid = 533,
    kGridArea = 534,
    kGridColumn = 535,
    kGridColumnGap = 536,
    kGridGap = 537,
    kGridRow = 538,
    kGridRowGap = 539,
    kGridTemplate = 540,
    kInset = 541,
    kInsetBlock = 542,
    kInsetInline = 543,
    kListStyle = 544,
    kMargin = 545,
    kMarginBlock = 546,
    kMarginInline = 547,
    kMarker = 548,
    kOffset = 549,
    kOutline = 550,
    kOverflow = 551,
    kOverscrollBehavior = 552,
    kPadding = 553,
    kPaddingBlock = 554,
    kPaddingInline = 555,
    kPageBreakAfter = 556,
    kPageBreakBefore = 557,
    kPageBreakInside = 558,
    kPlaceContent = 559,
    kPlaceItems = 560,
    kPlaceSelf = 561,
    kScrollMargin = 562,
    kScrollMarginBlock = 563,
    kScrollMarginInline = 564,
    kScrollPadding = 565,
    kScrollPaddingBlock = 566,
    kScrollPaddingInline = 567,
    kScrollStart = 568,
    kScrollStartTarget = 569,
    kScrollTimeline = 570,
    kTextDecoration = 571,
    kTextEmphasis = 572,
    kToggle = 573,
    kTransition = 574,
    kViewTimeline = 575,
    kWebkitColumnBreakAfter = 576,
    kWebkitColumnBreakBefore = 577,
    kWebkitColumnBreakInside = 578,
    kWebkitMask = 579,
    kWebkitMaskBoxImage = 580,
    kWebkitMaskPosition = 581,
    kWebkitMaskRepeat = 582,
    kWebkitTextStroke = 583,
    kWhiteSpace = 584,
    kAliasWebkitAppearance = 585,
    kAliasWebkitAppRegion = 586,
    kAliasWebkitBorderEndColor = 587,
    kAliasWebkitBorderEndStyle = 588,
    kAliasWebkitBorderEndWidth = 589,
    kAliasWebkitBorderStartColor = 590,
    kAliasWebkitBorderStartStyle = 591,
    kAliasWebkitBorderStartWidth = 592,
    kAliasWebkitBorderBeforeColor = 593,
    kAliasWebkitBorderBeforeStyle = 594,
    kAliasWebkitBorderBeforeWidth = 595,
    kAliasWebkitBorderAfterColor = 596,
    kAliasWebkitBorderAfterStyle = 597,
    kAliasWebkitBorderAfterWidth = 598,
    kAliasWebkitMarginEnd = 599,
    kAliasWebkitMarginStart = 600,
    kAliasWebkitMarginBefore = 601,
    kAliasWebkitMarginAfter = 602,
    kAliasWebkitPaddingEnd = 603,
    kAliasWebkitPaddingStart = 604,
    kAliasWebkitPaddingBefore = 605,
    kAliasWebkitPaddingAfter = 606,
    kAliasWebkitLogicalWidth = 607,
    kAliasWebkitLogicalHeight = 608,
    kAliasWebkitMinLogicalWidth = 609,
    kAliasWebkitMinLogicalHeight = 610,
    kAliasWebkitMaxLogicalWidth = 611,
    kAliasWebkitMaxLogicalHeight = 612,
    kAliasWebkitBorderAfter = 613,
    kAliasWebkitBorderBefore = 614,
    kAliasWebkitBorderEnd = 615,
    kAliasWebkitBorderStart = 616,
    kAliasEpubCaptionSide = 617,
    kAliasEpubTextCombine = 618,
    kAliasEpubTextEmphasis = 619,
    kAliasEpubTextEmphasisColor = 620,
    kAliasEpubTextEmphasisStyle = 621,
    kAliasEpubTextOrientation = 622,
    kAliasEpubTextTransform = 623,
    kAliasEpubWordBreak = 624,
    kAliasEpubWritingMode = 625,
    kAliasWebkitAlignContent = 626,
    kAliasWebkitAlignItems = 627,
    kAliasWebkitAlignSelf = 628,
    kAliasWebkitAnimation = 629,
    kAliasWebkitAlternativeAnimationWithTimeline = 630,
    kAliasWebkitAlternativeAnimationWithDelayStartEnd = 631,
    kAliasWebkitAnimationDelay = 632,
    kAliasWebkitAlternativeAnimationDelay = 633,
    kAliasWebkitAnimationDirection = 634,
    kAliasWebkitAnimationDuration = 635,
    kAliasWebkitAnimationFillMode = 636,
    kAliasWebkitAnimationIterationCount = 637,
    kAliasWebkitAnimationName = 638,
    kAliasWebkitAnimationPlayState = 639,
    kAliasWebkitAnimationTimingFunction = 640,
    kAliasWebkitBackfaceVisibility = 641,
    kAliasWebkitBackgroundClip = 642,
    kAliasWebkitBackgroundOrigin = 643,
    kAliasWebkitBackgroundSize = 644,
    kAliasWebkitBorderBottomLeftRadius = 645,
    kAliasWebkitBorderBottomRightRadius = 646,
    kAliasWebkitBorderRadius = 647,
    kAliasWebkitBorderTopLeftRadius = 648,
    kAliasWebkitBorderTopRightRadius = 649,
    kAliasWebkitBoxShadow = 650,
    kAliasWebkitBoxSizing = 651,
    kAliasWebkitClipPath = 652,
    kAliasWebkitColumnCount = 653,
    kAliasWebkitColumnGap = 654,
    kAliasWebkitColumnRule = 655,
    kAliasWebkitColumnRuleColor = 656,
    kAliasWebkitColumnRuleStyle = 657,
    kAliasWebkitColumnRuleWidth = 658,
    kAliasWebkitColumnSpan = 659,
    kAliasWebkitColumnWidth = 660,
    kAliasWebkitColumns = 661,
    kAliasWebkitFilter = 662,
    kAliasWebkitFlex = 663,
    kAliasWebkitFlexBasis = 664,
    kAliasWebkitFlexDirection = 665,
    kAliasWebkitFlexFlow = 666,
    kAliasWebkitFlexGrow = 667,
    kAliasWebkitFlexShrink = 668,
    kAliasWebkitFlexWrap = 669,
    kAliasWebkitFontFeatureSettings = 670,
    kAliasWebkitHyphenateCharacter = 671,
    kAliasWebkitJustifyContent = 672,
    kAliasWebkitOpacity = 673,
    kAliasWebkitOrder = 674,
    kAliasWebkitPerspective = 675,
    kAliasWebkitPerspectiveOrigin = 676,
    kAliasWebkitShapeImageThreshold = 677,
    kAliasWebkitShapeMargin = 678,
    kAliasWebkitShapeOutside = 679,
    kAliasWebkitTextEmphasis = 680,
    kAliasWebkitTextEmphasisColor = 681,
    kAliasWebkitTextEmphasisPosition = 682,
    kAliasWebkitTextEmphasisStyle = 683,
    kAliasWebkitTextSizeAdjust = 684,
    kAliasWebkitTransform = 685,
    kAliasWebkitTransformOrigin = 686,
    kAliasWebkitTransformStyle = 687,
    kAliasWebkitTransition = 688,
    kAliasWebkitTransitionDelay = 689,
    kAliasWebkitTransitionDuration = 690,
    kAliasWebkitTransitionProperty = 691,
    kAliasWebkitTransitionTimingFunction = 692,
    kAliasWebkitUserSelect = 693,
    kAliasWordWrap = 694,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitAppearance,
    CSSPropertyID::kAliasWebkitAppRegion,
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAlternativeAnimationWithTimeline,
    CSSPropertyID::kAliasWebkitAlternativeAnimationWithDelayStartEnd,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAlternativeAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitHyphenateCharacter,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextEmphasis,
    CSSPropertyID::kAliasWebkitTextEmphasisColor,
    CSSPropertyID::kAliasWebkitTextEmphasisPosition,
    CSSPropertyID::kAliasWebkitTextEmphasisStyle,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
};

const CSSPropertyID kCSSComputableProperties[] = {
    CSSPropertyID::kAccentColor,
    CSSPropertyID::kAlignContent,
    CSSPropertyID::kAlignItems,
    CSSPropertyID::kAlignSelf,
    CSSPropertyID::kAlignmentBaseline,
    CSSPropertyID::kAnchorDefault,
    CSSPropertyID::kAnchorName,
    CSSPropertyID::kAnimationComposition,
    CSSPropertyID::kAnimationDelay,
    CSSPropertyID::kAnimationDelayEnd,
    CSSPropertyID::kAnimationDelayStart,
    CSSPropertyID::kAnimationDirection,
    CSSPropertyID::kAnimationDuration,
    CSSPropertyID::kAnimationFillMode,
    CSSPropertyID::kAnimationIterationCount,
    CSSPropertyID::kAnimationName,
    CSSPropertyID::kAnimationPlayState,
    CSSPropertyID::kAnimationRangeEnd,
    CSSPropertyID::kAnimationRangeStart,
    CSSPropertyID::kAnimationTimeline,
    CSSPropertyID::kAnimationTimingFunction,
    CSSPropertyID::kAppRegion,
    CSSPropertyID::kAppearance,
    CSSPropertyID::kBackdropFilter,
    CSSPropertyID::kBackfaceVisibility,
    CSSPropertyID::kBackgroundAttachment,
    CSSPropertyID::kBackgroundBlendMode,
    CSSPropertyID::kBackgroundClip,
    CSSPropertyID::kBackgroundColor,
    CSSPropertyID::kBackgroundImage,
    CSSPropertyID::kBackgroundOrigin,
    CSSPropertyID::kBackgroundPosition,
    CSSPropertyID::kBackgroundRepeat,
    CSSPropertyID::kBackgroundSize,
    CSSPropertyID::kBaselineShift,
    CSSPropertyID::kBaselineSource,
    CSSPropertyID::kBlockSize,
    CSSPropertyID::kBorderBlockEndColor,
    CSSPropertyID::kBorderBlockEndStyle,
    CSSPropertyID::kBorderBlockEndWidth,
    CSSPropertyID::kBorderBlockStartColor,
    CSSPropertyID::kBorderBlockStartStyle,
    CSSPropertyID::kBorderBlockStartWidth,
    CSSPropertyID::kBorderBottomColor,
    CSSPropertyID::kBorderBottomLeftRadius,
    CSSPropertyID::kBorderBottomRightRadius,
    CSSPropertyID::kBorderBottomStyle,
    CSSPropertyID::kBorderBottomWidth,
    CSSPropertyID::kBorderCollapse,
    CSSPropertyID::kBorderEndEndRadius,
    CSSPropertyID::kBorderEndStartRadius,
    CSSPropertyID::kBorderImageOutset,
    CSSPropertyID::kBorderImageRepeat,
    CSSPropertyID::kBorderImageSlice,
    CSSPropertyID::kBorderImageSource,
    CSSPropertyID::kBorderImageWidth,
    CSSPropertyID::kBorderInlineEndColor,
    CSSPropertyID::kBorderInlineEndStyle,
    CSSPropertyID::kBorderInlineEndWidth,
    CSSPropertyID::kBorderInlineStartColor,
    CSSPropertyID::kBorderInlineStartStyle,
    CSSPropertyID::kBorderInlineStartWidth,
    CSSPropertyID::kBorderLeftColor,
    CSSPropertyID::kBorderLeftStyle,
    CSSPropertyID::kBorderLeftWidth,
    CSSPropertyID::kBorderRightColor,
    CSSPropertyID::kBorderRightStyle,
    CSSPropertyID::kBorderRightWidth,
    CSSPropertyID::kBorderStartEndRadius,
    CSSPropertyID::kBorderStartStartRadius,
    CSSPropertyID::kBorderTopColor,
    CSSPropertyID::kBorderTopLeftRadius,
    CSSPropertyID::kBorderTopRightRadius,
    CSSPropertyID::kBorderTopStyle,
    CSSPropertyID::kBorderTopWidth,
    CSSPropertyID::kBottom,
    CSSPropertyID::kBoxShadow,
    CSSPropertyID::kBoxSizing,
    CSSPropertyID::kBreakAfter,
    CSSPropertyID::kBreakBefore,
    CSSPropertyID::kBreakInside,
    CSSPropertyID::kBufferedRendering,
    CSSPropertyID::kCaptionSide,
    CSSPropertyID::kCaretColor,
    CSSPropertyID::kClear,
    CSSPropertyID::kClip,
    CSSPropertyID::kClipPath,
    CSSPropertyID::kClipRule,
    CSSPropertyID::kColor,
    CSSPropertyID::kColorInterpolation,
    CSSPropertyID::kColorInterpolationFilters,
    CSSPropertyID::kColorRendering,
    CSSPropertyID::kColumnCount,
    CSSPropertyID::kColumnGap,
    CSSPropertyID::kColumnRuleColor,
    CSSPropertyID::kColumnRuleStyle,
    CSSPropertyID::kColumnRuleWidth,
    CSSPropertyID::kColumnSpan,
    CSSPropertyID::kColumnWidth,
    CSSPropertyID::kContainIntrinsicBlockSize,
    CSSPropertyID::kContainIntrinsicHeight,
    CSSPropertyID::kContainIntrinsicInlineSize,
    CSSPropertyID::kContainIntrinsicSize,
    CSSPropertyID::kContainIntrinsicWidth,
    CSSPropertyID::kContainerName,
    CSSPropertyID::kContainerType,
    CSSPropertyID::kContent,
    CSSPropertyID::kCursor,
    CSSPropertyID::kCx,
    CSSPropertyID::kCy,
    CSSPropertyID::kD,
    CSSPropertyID::kDirection,
    CSSPropertyID::kDisplay,
    CSSPropertyID::kDominantBaseline,
    CSSPropertyID::kDynamicRangeLimit,
    CSSPropertyID::kEmptyCells,
    CSSPropertyID::kFill,
    CSSPropertyID::kFillOpacity,
    CSSPropertyID::kFillRule,
    CSSPropertyID::kFilter,
    CSSPropertyID::kFlexBasis,
    CSSPropertyID::kFlexDirection,
    CSSPropertyID::kFlexGrow,
    CSSPropertyID::kFlexShrink,
    CSSPropertyID::kFlexWrap,
    CSSPropertyID::kFloat,
    CSSPropertyID::kFloodColor,
    CSSPropertyID::kFloodOpacity,
    CSSPropertyID::kFontFamily,
    CSSPropertyID::kFontKerning,
    CSSPropertyID::kFontOpticalSizing,
    CSSPropertyID::kFontPalette,
    CSSPropertyID::kFontSize,
    CSSPropertyID::kFontSizeAdjust,
    CSSPropertyID::kFontStretch,
    CSSPropertyID::kFontStyle,
    CSSPropertyID::kFontSynthesisSmallCaps,
    CSSPropertyID::kFontSynthesisStyle,
    CSSPropertyID::kFontSynthesisWeight,
    CSSPropertyID::kFontVariant,
    CSSPropertyID::kFontVariantAlternates,
    CSSPropertyID::kFontVariantCaps,
    CSSPropertyID::kFontVariantEastAsian,
    CSSPropertyID::kFontVariantLigatures,
    CSSPropertyID::kFontVariantNumeric,
    CSSPropertyID::kFontVariantPosition,
    CSSPropertyID::kFontWeight,
    CSSPropertyID::kFormSizing,
    CSSPropertyID::kGridAutoColumns,
    CSSPropertyID::kGridAutoFlow,
    CSSPropertyID::kGridAutoRows,
    CSSPropertyID::kGridColumnEnd,
    CSSPropertyID::kGridColumnStart,
    CSSPropertyID::kGridRowEnd,
    CSSPropertyID::kGridRowStart,
    CSSPropertyID::kGridTemplateAreas,
    CSSPropertyID::kGridTemplateColumns,
    CSSPropertyID::kGridTemplateRows,
    CSSPropertyID::kHeight,
    CSSPropertyID::kHyphenateCharacter,
    CSSPropertyID::kHyphenateLimitChars,
    CSSPropertyID::kHyphens,
    CSSPropertyID::kImageOrientation,
    CSSPropertyID::kImageRendering,
    CSSPropertyID::kInitialLetter,
    CSSPropertyID::kInlineSize,
    CSSPropertyID::kInsetBlockEnd,
    CSSPropertyID::kInsetBlockStart,
    CSSPropertyID::kInsetInlineEnd,
    CSSPropertyID::kInsetInlineStart,
    CSSPropertyID::kIsolation,
    CSSPropertyID::kJustifyContent,
    CSSPropertyID::kJustifyItems,
    CSSPropertyID::kJustifySelf,
    CSSPropertyID::kLeft,
    CSSPropertyID::kLetterSpacing,
    CSSPropertyID::kLightingColor,
    CSSPropertyID::kLineBreak,
    CSSPropertyID::kLineHeight,
    CSSPropertyID::kListStyleImage,
    CSSPropertyID::kListStylePosition,
    CSSPropertyID::kListStyleType,
    CSSPropertyID::kMarginBlockEnd,
    CSSPropertyID::kMarginBlockStart,
    CSSPropertyID::kMarginBottom,
    CSSPropertyID::kMarginInlineEnd,
    CSSPropertyID::kMarginInlineStart,
    CSSPropertyID::kMarginLeft,
    CSSPropertyID::kMarginRight,
    CSSPropertyID::kMarginTop,
    CSSPropertyID::kMarkerEnd,
    CSSPropertyID::kMarkerMid,
    CSSPropertyID::kMarkerStart,
    CSSPropertyID::kMaskType,
    CSSPropertyID::kMathDepth,
    CSSPropertyID::kMathShift,
    CSSPropertyID::kMathStyle,
    CSSPropertyID::kMaxBlockSize,
    CSSPropertyID::kMaxHeight,
    CSSPropertyID::kMaxInlineSize,
    CSSPropertyID::kMaxWidth,
    CSSPropertyID::kMinBlockSize,
    CSSPropertyID::kMinHeight,
    CSSPropertyID::kMinInlineSize,
    CSSPropertyID::kMinWidth,
    CSSPropertyID::kMixBlendMode,
    CSSPropertyID::kObjectFit,
    CSSPropertyID::kObjectPosition,
    CSSPropertyID::kObjectViewBox,
    CSSPropertyID::kOffsetAnchor,
    CSSPropertyID::kOffsetDistance,
    CSSPropertyID::kOffsetPath,
    CSSPropertyID::kOffsetPosition,
    CSSPropertyID::kOffsetRotate,
    CSSPropertyID::kOpacity,
    CSSPropertyID::kOrder,
    CSSPropertyID::kOrphans,
    CSSPropertyID::kOutlineColor,
    CSSPropertyID::kOutlineOffset,
    CSSPropertyID::kOutlineStyle,
    CSSPropertyID::kOutlineWidth,
    CSSPropertyID::kOverflowAnchor,
    CSSPropertyID::kOverflowBlock,
    CSSPropertyID::kOverflowClipMargin,
    CSSPropertyID::kOverflowInline,
    CSSPropertyID::kOverflowWrap,
    CSSPropertyID::kOverflowX,
    CSSPropertyID::kOverflowY,
    CSSPropertyID::kOverlay,
    CSSPropertyID::kOverscrollBehaviorBlock,
    CSSPropertyID::kOverscrollBehaviorInline,
    CSSPropertyID::kPaddingBlockEnd,
    CSSPropertyID::kPaddingBlockStart,
    CSSPropertyID::kPaddingBottom,
    CSSPropertyID::kPaddingInlineEnd,
    CSSPropertyID::kPaddingInlineStart,
    CSSPropertyID::kPaddingLeft,
    CSSPropertyID::kPaddingRight,
    CSSPropertyID::kPaddingTop,
    CSSPropertyID::kPaintOrder,
    CSSPropertyID::kPerspective,
    CSSPropertyID::kPerspectiveOrigin,
    CSSPropertyID::kPointerEvents,
    CSSPropertyID::kPopoverHideDelay,
    CSSPropertyID::kPopoverShowDelay,
    CSSPropertyID::kPosition,
    CSSPropertyID::kPositionFallback,
    CSSPropertyID::kPositionFallbackBounds,
    CSSPropertyID::kR,
    CSSPropertyID::kResize,
    CSSPropertyID::kRight,
    CSSPropertyID::kRotate,
    CSSPropertyID::kRowGap,
    CSSPropertyID::kRubyPosition,
    CSSPropertyID::kRx,
    CSSPropertyID::kRy,
    CSSPropertyID::kScale,
    CSSPropertyID::kScrollBehavior,
    CSSPropertyID::kScrollCustomization,
    CSSPropertyID::kScrollMarginBlockEnd,
    CSSPropertyID::kScrollMarginBlockStart,
    CSSPropertyID::kScrollMarginInlineEnd,
    CSSPropertyID::kScrollMarginInlineStart,
    CSSPropertyID::kScrollPaddingBlockEnd,
    CSSPropertyID::kScrollPaddingBlockStart,
    CSSPropertyID::kScrollPaddingInlineEnd,
    CSSPropertyID::kScrollPaddingInlineStart,
    CSSPropertyID::kScrollStartBlock,
    CSSPropertyID::kScrollStartInline,
    CSSPropertyID::kScrollStartTargetBlock,
    CSSPropertyID::kScrollStartTargetInline,
    CSSPropertyID::kScrollStartTargetX,
    CSSPropertyID::kScrollStartTargetY,
    CSSPropertyID::kScrollStartX,
    CSSPropertyID::kScrollStartY,
    CSSPropertyID::kScrollTimelineAxis,
    CSSPropertyID::kScrollTimelineName,
    CSSPropertyID::kScrollbarColor,
    CSSPropertyID::kScrollbarGutter,
    CSSPropertyID::kScrollbarWidth,
    CSSPropertyID::kShapeImageThreshold,
    CSSPropertyID::kShapeMargin,
    CSSPropertyID::kShapeOutside,
    CSSPropertyID::kShapeRendering,
    CSSPropertyID::kSpeak,
    CSSPropertyID::kStopColor,
    CSSPropertyID::kStopOpacity,
    CSSPropertyID::kStroke,
    CSSPropertyID::kStrokeDasharray,
    CSSPropertyID::kStrokeDashoffset,
    CSSPropertyID::kStrokeLinecap,
    CSSPropertyID::kStrokeLinejoin,
    CSSPropertyID::kStrokeMiterlimit,
    CSSPropertyID::kStrokeOpacity,
    CSSPropertyID::kStrokeWidth,
    CSSPropertyID::kTabSize,
    CSSPropertyID::kTableLayout,
    CSSPropertyID::kTextAlign,
    CSSPropertyID::kTextAlignLast,
    CSSPropertyID::kTextAnchor,
    CSSPropertyID::kTextAutospace,
    CSSPropertyID::kTextBoxTrim,
    CSSPropertyID::kTextDecoration,
    CSSPropertyID::kTextDecorationColor,
    CSSPropertyID::kTextDecorationLine,
    CSSPropertyID::kTextDecorationSkipInk,
    CSSPropertyID::kTextDecorationStyle,
    CSSPropertyID::kTextEmphasisColor,
    CSSPropertyID::kTextEmphasisPosition,
    CSSPropertyID::kTextEmphasisStyle,
    CSSPropertyID::kTextIndent,
    CSSPropertyID::kTextOverflow,
    CSSPropertyID::kTextRendering,
    CSSPropertyID::kTextShadow,
    CSSPropertyID::kTextSizeAdjust,
    CSSPropertyID::kTextTransform,
    CSSPropertyID::kTextUnderlinePosition,
    CSSPropertyID::kTextWrap,
    CSSPropertyID::kTimelineScope,
    CSSPropertyID::kToggleGroup,
    CSSPropertyID::kToggleRoot,
    CSSPropertyID::kToggleTrigger,
    CSSPropertyID::kToggleVisibility,
    CSSPropertyID::kTop,
    CSSPropertyID::kTouchAction,
    CSSPropertyID::kTransform,
    CSSPropertyID::kTransformOrigin,
    CSSPropertyID::kTransformStyle,
    CSSPropertyID::kTransitionBehavior,
    CSSPropertyID::kTransitionDelay,
    CSSPropertyID::kTransitionDuration,
    CSSPropertyID::kTransitionProperty,
    CSSPropertyID::kTransitionTimingFunction,
    CSSPropertyID::kTranslate,
    CSSPropertyID::kUnicodeBidi,
    CSSPropertyID::kUserSelect,
    CSSPropertyID::kVectorEffect,
    CSSPropertyID::kVerticalAlign,
    CSSPropertyID::kViewTimelineAxis,
    CSSPropertyID::kViewTimelineInset,
    CSSPropertyID::kViewTimelineName,
    CSSPropertyID::kViewTransitionName,
    CSSPropertyID::kVisibility,
    CSSPropertyID::kWhiteSpaceCollapse,
    CSSPropertyID::kWidows,
    CSSPropertyID::kWidth,
    CSSPropertyID::kWillChange,
    CSSPropertyID::kWordBreak,
    CSSPropertyID::kWordSpacing,
    CSSPropertyID::kWritingMode,
    CSSPropertyID::kX,
    CSSPropertyID::kY,
    CSSPropertyID::kZIndex,
    CSSPropertyID::kZoom,
    CSSPropertyID::kWebkitBorderHorizontalSpacing,
    CSSPropertyID::kWebkitBorderImage,
    CSSPropertyID::kWebkitBorderVerticalSpacing,
    CSSPropertyID::kWebkitBoxAlign,
    CSSPropertyID::kWebkitBoxDecorationBreak,
    CSSPropertyID::kWebkitBoxDirection,
    CSSPropertyID::kWebkitBoxFlex,
    CSSPropertyID::kWebkitBoxOrdinalGroup,
    CSSPropertyID::kWebkitBoxOrient,
    CSSPropertyID::kWebkitBoxPack,
    CSSPropertyID::kWebkitBoxReflect,
    CSSPropertyID::kWebkitFontSmoothing,
    CSSPropertyID::kWebkitLineBreak,
    CSSPropertyID::kWebkitLineClamp,
    CSSPropertyID::kWebkitLocale,
    CSSPropertyID::kWebkitMaskBoxImage,
    CSSPropertyID::kWebkitMaskBoxImageOutset,
    CSSPropertyID::kWebkitMaskBoxImageRepeat,
    CSSPropertyID::kWebkitMaskBoxImageSlice,
    CSSPropertyID::kWebkitMaskBoxImageSource,
    CSSPropertyID::kWebkitMaskBoxImageWidth,
    CSSPropertyID::kWebkitMaskClip,
    CSSPropertyID::kWebkitMaskComposite,
    CSSPropertyID::kWebkitMaskImage,
    CSSPropertyID::kWebkitMaskOrigin,
    CSSPropertyID::kWebkitMaskPosition,
    CSSPropertyID::kWebkitMaskRepeat,
    CSSPropertyID::kWebkitMaskSize,
    CSSPropertyID::kWebkitPrintColorAdjust,
    CSSPropertyID::kWebkitRtlOrdering,
    CSSPropertyID::kWebkitTapHighlightColor,
    CSSPropertyID::kWebkitTextCombine,
    CSSPropertyID::kWebkitTextDecorationsInEffect,
    CSSPropertyID::kWebkitTextFillColor,
    CSSPropertyID::kWebkitTextOrientation,
    CSSPropertyID::kWebkitTextSecurity,
    CSSPropertyID::kWebkitTextStrokeColor,
    CSSPropertyID::kWebkitTextStrokeWidth,
    CSSPropertyID::kWebkitUserDrag,
    CSSPropertyID::kWebkitUserModify,
    CSSPropertyID::kWebkitWritingMode,
};

// The lowest CSSPropertyID excluding kInvalid and kVariable.
const int kIntFirstCSSProperty = 2;
const CSSPropertyID kFirstCSSProperty =
    static_cast<CSSPropertyID>(kIntFirstCSSProperty);

// The number of unresolved CSS properties excluding kInvalid and kVariable.
const int kNumCSSProperties = 693;

// The highest resolved CSSPropertyID.
const int kIntLastCSSProperty = 584;
const CSSPropertyID kLastCSSProperty =
    static_cast<CSSPropertyID>(kIntLastCSSProperty);

// The highest unresolved CSSPropertyID.
const CSSPropertyID kLastUnresolvedCSSProperty =
    static_cast<CSSPropertyID>(694);

const CSSPropertyID kFirstHighPriorityCSSProperty = kFirstCSSProperty;
const CSSPropertyID kLastHighPriorityCSSProperty = CSSPropertyID::kZoom;

// 1 + <The highest unresolved CSSPropertyID>.
const int kNumCSSPropertyIDs = static_cast<int>(kLastUnresolvedCSSProperty) + 1;

const size_t kMaxCSSPropertyNameLength = 48;
constexpr size_t kCSSPropertyIDBitLength = 10;
constexpr size_t kMaxShorthandExpansion = 17;

static_assert((static_cast<size_t>(1) << kCSSPropertyIDBitLength) >
                  static_cast<size_t>(kLastUnresolvedCSSProperty),
              "kCSSPropertyIDBitLength has enough bits");

static_assert(CSSPropertyID::kColor == kFirstHighPriorityCSSProperty,
              "CSSPropertyID::kColor is the first high-priority property");
static_assert(CSSPropertyID::kZoom == kLastHighPriorityCSSProperty,
              "CSSPropertyID::kZoom is the last high-priority property");
static_assert((static_cast<int>(kLastHighPriorityCSSProperty) -
               static_cast<int>(kFirstHighPriorityCSSProperty)) == 31,
              "There should a low number of high-priority properties");

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, kFirstCSSProperty);
    DCHECK_LE(id, kLastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

constexpr bool IsHighPriority(CSSPropertyID id) {
  return id >= kFirstHighPriorityCSSProperty &&
      id <= kLastHighPriorityCSSProperty;
}

inline bool IsCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= kFirstCSSProperty && id <= kLastUnresolvedCSSProperty;
}

inline bool IsValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID ConvertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

int CORE_EXPORT ResolveCSSPropertyAlias(int value);

inline bool IsPropertyAlias(CSSPropertyID id) {
  return static_cast<int>(id) >= 585;
}

inline CSSPropertyID ResolveCSSPropertyID(CSSPropertyID id)
{
  int int_id = static_cast<int>(id);
  if (IsPropertyAlias(id))
    int_id = ResolveCSSPropertyAlias(int_id);
  return ConvertToCSSPropertyID(int_id);
}

CSSPropertyID CORE_EXPORT UnresolvedCSSPropertyID(const ExecutionContext*,
                                                  const WTF::String&);

CSSPropertyID CORE_EXPORT CssPropertyID(const ExecutionContext*,
                                        const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();

 public:
  class Iterator {
    STACK_ALLOCATED();
   public:
    Iterator(int id) : id_(id) {}
    CSSPropertyID operator*() const { return ConvertToCSSPropertyID(id_); }
    Iterator& operator++() {
      id_++;
      return *this;
    }
    bool operator!=(const Iterator& i) const { return id_ != i.id_; }

   private:
    int id_;
  };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

mojom::blink::CSSSampleId CORE_EXPORT GetCSSSampleId(CSSPropertyID id);

}  // namespace blink

// Even though these hash traits are not affected by code generation,
// it's important to define them in the same file as CSSPropertyID to avoid
// hard-to-diagnose problems from neglecting to include said traits.
// See https://crbug.com/1246970.
namespace WTF {
template <>
struct HashTraits<blink::CSSPropertyID>
    : EnumHashTraits<blink::CSSPropertyID,
                     blink::CSSPropertyID::kInvalid,
                     blink::kNumCSSPropertyIDs> {};
}  // namespace WTF

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
