// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kStylistic = 56,
  kHistoricalForms = 57,
  kStyleset = 58,
  kCharacterVariant = 59,
  kSwash = 60,
  kOrnaments = 61,
  kAnnotation = 62,
  kNormal = 63,
  kBold = 64,
  kBolder = 65,
  kLighter = 66,
  kWeight = 67,
  kUltraCondensed = 68,
  kExtraCondensed = 69,
  kCondensed = 70,
  kSemiCondensed = 71,
  kSemiExpanded = 72,
  kExpanded = 73,
  kExtraExpanded = 74,
  kUltraExpanded = 75,
  kXxSmall = 76,
  kXSmall = 77,
  kSmall = 78,
  kMedium = 79,
  kLarge = 80,
  kXLarge = 81,
  kXxLarge = 82,
  kXxxLarge = 83,
  kSmaller = 84,
  kLarger = 85,
  kWebkitXxxLarge = 86,
  kExHeight = 87,
  kCapHeight = 88,
  kChWidth = 89,
  kIcWidth = 90,
  kSerif = 91,
  kSansSerif = 92,
  kCursive = 93,
  kFantasy = 94,
  kMonospace = 95,
  kSystemUi = 96,
  kWebkitBody = 97,
  kMath = 98,
  kSwap = 99,
  kFallback = 100,
  kOptional = 101,
  kFontTech = 102,
  kFontFormat = 103,
  kPaletteMix = 104,
  kAqua = 105,
  kBlack = 106,
  kBlue = 107,
  kFuchsia = 108,
  kGray = 109,
  kGreen = 110,
  kLime = 111,
  kMaroon = 112,
  kNavy = 113,
  kOlive = 114,
  kOrange = 115,
  kPurple = 116,
  kRed = 117,
  kSilver = 118,
  kTeal = 119,
  kWhite = 120,
  kYellow = 121,
  kTransparent = 122,
  kWebkitLink = 123,
  kWebkitActivelink = 124,
  kAccentcolor = 125,
  kAccentcolortext = 126,
  kActiveborder = 127,
  kActivecaption = 128,
  kActivetext = 129,
  kAppworkspace = 130,
  kBackground = 131,
  kButtonborder = 132,
  kButtonface = 133,
  kButtonhighlight = 134,
  kButtonshadow = 135,
  kButtontext = 136,
  kCanvas = 137,
  kCanvastext = 138,
  kCaptiontext = 139,
  kField = 140,
  kFieldtext = 141,
  kGraytext = 142,
  kHighlight = 143,
  kHighlighttext = 144,
  kInactiveborder = 145,
  kInactivecaption = 146,
  kInactivecaptiontext = 147,
  kInfobackground = 148,
  kInfotext = 149,
  kLinktext = 150,
  kMark = 151,
  kMarktext = 152,
  kMenutext = 153,
  kSelecteditem = 154,
  kSelecteditemtext = 155,
  kScrollbar = 156,
  kThreeddarkshadow = 157,
  kThreedface = 158,
  kThreedhighlight = 159,
  kThreedlightshadow = 160,
  kThreedshadow = 161,
  kVisitedtext = 162,
  kWindow = 163,
  kWindowframe = 164,
  kWindowtext = 165,
  kInternalActiveListBoxSelection = 166,
  kInternalActiveListBoxSelectionText = 167,
  kInternalInactiveListBoxSelection = 168,
  kInternalInactiveListBoxSelectionText = 169,
  kWebkitFocusRingColor = 170,
  kCurrentcolor = 171,
  kGrey = 172,
  kInternalQuirkInherit = 173,
  kInternalSpellingErrorColor = 174,
  kInternalGrammarErrorColor = 175,
  kAliceblue = 176,
  kAntiquewhite = 177,
  kAquamarine = 178,
  kAzure = 179,
  kBeige = 180,
  kBisque = 181,
  kBlanchedalmond = 182,
  kBlueviolet = 183,
  kBrown = 184,
  kBurlywood = 185,
  kCadetblue = 186,
  kChartreuse = 187,
  kChocolate = 188,
  kCoral = 189,
  kCornflowerblue = 190,
  kCornsilk = 191,
  kCrimson = 192,
  kCyan = 193,
  kDarkblue = 194,
  kDarkcyan = 195,
  kDarkgoldenrod = 196,
  kDarkgray = 197,
  kDarkgreen = 198,
  kDarkgrey = 199,
  kDarkkhaki = 200,
  kDarkmagenta = 201,
  kDarkolivegreen = 202,
  kDarkorange = 203,
  kDarkorchid = 204,
  kDarkred = 205,
  kDarksalmon = 206,
  kDarkseagreen = 207,
  kDarkslateblue = 208,
  kDarkslategray = 209,
  kDarkslategrey = 210,
  kDarkturquoise = 211,
  kDarkviolet = 212,
  kDeeppink = 213,
  kDeepskyblue = 214,
  kDimgray = 215,
  kDimgrey = 216,
  kDodgerblue = 217,
  kFirebrick = 218,
  kFloralwhite = 219,
  kForestgreen = 220,
  kGainsboro = 221,
  kGhostwhite = 222,
  kGold = 223,
  kGoldenrod = 224,
  kGreenyellow = 225,
  kHoneydew = 226,
  kHotpink = 227,
  kIndianred = 228,
  kIndigo = 229,
  kIvory = 230,
  kKhaki = 231,
  kLavender = 232,
  kLavenderblush = 233,
  kLawngreen = 234,
  kLemonchiffon = 235,
  kLightblue = 236,
  kLightcoral = 237,
  kLightcyan = 238,
  kLightgoldenrodyellow = 239,
  kLightgray = 240,
  kLightgreen = 241,
  kLightgrey = 242,
  kLightpink = 243,
  kLightsalmon = 244,
  kLightseagreen = 245,
  kLightskyblue = 246,
  kLightslategray = 247,
  kLightslategrey = 248,
  kLightsteelblue = 249,
  kLightyellow = 250,
  kLimegreen = 251,
  kLinen = 252,
  kMagenta = 253,
  kMediumaquamarine = 254,
  kMediumblue = 255,
  kMediumorchid = 256,
  kMediumpurple = 257,
  kMediumseagreen = 258,
  kMediumslateblue = 259,
  kMediumspringgreen = 260,
  kMediumturquoise = 261,
  kMediumvioletred = 262,
  kMidnightblue = 263,
  kMintcream = 264,
  kMistyrose = 265,
  kMoccasin = 266,
  kNavajowhite = 267,
  kOldlace = 268,
  kOlivedrab = 269,
  kOrangered = 270,
  kOrchid = 271,
  kPalegoldenrod = 272,
  kPalegreen = 273,
  kPaleturquoise = 274,
  kPalevioletred = 275,
  kPapayawhip = 276,
  kPeachpuff = 277,
  kPeru = 278,
  kPink = 279,
  kPlum = 280,
  kPowderblue = 281,
  kRebeccapurple = 282,
  kRosybrown = 283,
  kRoyalblue = 284,
  kSaddlebrown = 285,
  kSalmon = 286,
  kSandybrown = 287,
  kSeagreen = 288,
  kSeashell = 289,
  kSienna = 290,
  kSkyblue = 291,
  kSlateblue = 292,
  kSlategray = 293,
  kSlategrey = 294,
  kSnow = 295,
  kSpringgreen = 296,
  kSteelblue = 297,
  kTan = 298,
  kThistle = 299,
  kTomato = 300,
  kTurquoise = 301,
  kViolet = 302,
  kWheat = 303,
  kWhitesmoke = 304,
  kYellowgreen = 305,
  kRepeat = 306,
  kRepeatX = 307,
  kRepeatY = 308,
  kNoRepeat = 309,
  kClear = 310,
  kCopy = 311,
  kSourceOver = 312,
  kSourceIn = 313,
  kSourceOut = 314,
  kSourceAtop = 315,
  kDestinationOver = 316,
  kDestinationIn = 317,
  kDestinationOut = 318,
  kDestinationAtop = 319,
  kXor = 320,
  kPlusLighter = 321,
  kBaseline = 322,
  kMiddle = 323,
  kSub = 324,
  kSuper = 325,
  kTextTop = 326,
  kTextBottom = 327,
  kTop = 328,
  kBottom = 329,
  kWebkitBaselineMiddle = 330,
  kWebkitAuto = 331,
  kLeft = 332,
  kRight = 333,
  kCenter = 334,
  kJustify = 335,
  kWebkitLeft = 336,
  kWebkitRight = 337,
  kWebkitCenter = 338,
  kWebkitMatchParent = 339,
  kInternalCenter = 340,
  kInlineStart = 341,
  kInlineEnd = 342,
  kOutside = 343,
  kInside = 344,
  kDisc = 345,
  kCircle = 346,
  kSquare = 347,
  kDisclosureOpen = 348,
  kDisclosureClosed = 349,
  kDecimal = 350,
  kInline = 351,
  kBlock = 352,
  kFlowRoot = 353,
  kFlow = 354,
  kTable = 355,
  kFlex = 356,
  kGrid = 357,
  kContents = 358,
  kTableRowGroup = 359,
  kTableHeaderGroup = 360,
  kTableFooterGroup = 361,
  kTableRow = 362,
  kTableColumnGroup = 363,
  kTableColumn = 364,
  kTableCell = 365,
  kTableCaption = 366,
  kInlineBlock = 367,
  kInlineTable = 368,
  kInlineFlex = 369,
  kInlineGrid = 370,
  kWebkitBox = 371,
  kWebkitInlineBox = 372,
  kWebkitFlex = 373,
  kWebkitInlineFlex = 374,
  kLayout = 375,
  kInlineLayout = 376,
  kListItem = 377,
  kAuto = 378,
  kCrosshair = 379,
  kDefault = 380,
  kPointer = 381,
  kMove = 382,
  kVerticalText = 383,
  kCell = 384,
  kContextMenu = 385,
  kAlias = 386,
  kProgress = 387,
  kNoDrop = 388,
  kNotAllowed = 389,
  kZoomIn = 390,
  kZoomOut = 391,
  kEResize = 392,
  kNeResize = 393,
  kNwResize = 394,
  kNResize = 395,
  kSeResize = 396,
  kSwResize = 397,
  kSResize = 398,
  kWResize = 399,
  kEwResize = 400,
  kNsResize = 401,
  kNeswResize = 402,
  kNwseResize = 403,
  kColResize = 404,
  kRowResize = 405,
  kText = 406,
  kWait = 407,
  kHelp = 408,
  kAllScroll = 409,
  kGrab = 410,
  kGrabbing = 411,
  kWebkitGrab = 412,
  kWebkitGrabbing = 413,
  kWebkitZoomIn = 414,
  kWebkitZoomOut = 415,
  kLtr = 416,
  kRtl = 417,
  kCapitalize = 418,
  kUppercase = 419,
  kLowercase = 420,
  kMathAuto = 421,
  kVisible = 422,
  kCollapse = 423,
  kPreserve = 424,
  kPreserveBreaks = 425,
  kPretty = 426,
  kA3 = 427,
  kA4 = 428,
  kA5 = 429,
  kAbove = 430,
  kAbsolute = 431,
  kAlways = 432,
  kAvoid = 433,
  kB4 = 434,
  kB5 = 435,
  kBelow = 436,
  kBidiOverride = 437,
  kBlink = 438,
  kBoth = 439,
  kBreakSpaces = 440,
  kCloseQuote = 441,
  kEmbed = 442,
  kFixed = 443,
  kHand = 444,
  kHide = 445,
  kIsolate = 446,
  kIsolateOverride = 447,
  kPlaintext = 448,
  kWebkitIsolate = 449,
  kWebkitIsolateOverride = 450,
  kWebkitPlaintext = 451,
  kJisB5 = 452,
  kJisB4 = 453,
  kLandscape = 454,
  kLedger = 455,
  kLegal = 456,
  kLetter = 457,
  kLineThrough = 458,
  kLocal = 459,
  kNoCloseQuote = 460,
  kNoOpenQuote = 461,
  kNowrap = 462,
  kOpenQuote = 463,
  kOverlay = 464,
  kOverline = 465,
  kPortrait = 466,
  kPre = 467,
  kPreLine = 468,
  kPreWrap = 469,
  kRelative = 470,
  kScroll = 471,
  kSeparate = 472,
  kShow = 473,
  kStatic = 474,
  kThick = 475,
  kThin = 476,
  kUnderline = 477,
  kView = 478,
  kWavy = 479,
  kCompact = 480,
  kStretch = 481,
  kStart = 482,
  kEnd = 483,
  kClone = 484,
  kSlice = 485,
  kReverse = 486,
  kHorizontal = 487,
  kVertical = 488,
  kInlineAxis = 489,
  kBlockAxis = 490,
  kFlexStart = 491,
  kFlexEnd = 492,
  kSpaceBetween = 493,
  kSpaceAround = 494,
  kSpaceEvenly = 495,
  kUnsafe = 496,
  kSafe = 497,
  kRow = 498,
  kRowReverse = 499,
  kColumn = 500,
  kColumnReverse = 501,
  kWrap = 502,
  kWrapReverse = 503,
  kAutoFlow = 504,
  kDense = 505,
  kReadOnly = 506,
  kReadWrite = 507,
  kReadWritePlaintextOnly = 508,
  kElement = 509,
  kWebkitMinContent = 510,
  kWebkitMaxContent = 511,
  kWebkitFillAvailable = 512,
  kWebkitFitContent = 513,
  kMinContent = 514,
  kMaxContent = 515,
  kFitContent = 516,
  kNoAutospace = 517,
  kClip = 518,
  kEllipsis = 519,
  kSpellingError = 520,
  kGrammarError = 521,
  kFromFont = 522,
  kBreakAll = 523,
  kKeepAll = 524,
  kAutoPhrase = 525,
  kBreakWord = 526,
  kSpace = 527,
  kLoose = 528,
  kStrict = 529,
  kAfterWhiteSpace = 530,
  kAnywhere = 531,
  kManual = 532,
  kCheckbox = 533,
  kRadio = 534,
  kButton = 535,
  kListbox = 536,
  kInternalMediaControl = 537,
  kMenulist = 538,
  kMenulistButton = 539,
  kMeter = 540,
  kProgressBar = 541,
  kSearchfield = 542,
  kTextfield = 543,
  kTextarea = 544,
  kInnerSpinButton = 545,
  kPushButton = 546,
  kSquareButton = 547,
  kSliderHorizontal = 548,
  kSearchfieldCancelButton = 549,
  kMediaSlider = 550,
  kMediaSliderthumb = 551,
  kMediaVolumeSlider = 552,
  kMediaVolumeSliderthumb = 553,
  kSliderthumbHorizontal = 554,
  kSliderthumbVertical = 555,
  kSliderVertical = 556,
  kRound = 557,
  kBorder = 558,
  kBorderBox = 559,
  kContent = 560,
  kContentBox = 561,
  kPadding = 562,
  kPaddingBox = 563,
  kMarginBox = 564,
  kContain = 565,
  kCover = 566,
  kLogical = 567,
  kVisual = 568,
  kReplace = 569,
  kAccumulate = 570,
  kAlternate = 571,
  kAlternateReverse = 572,
  kForwards = 573,
  kBackwards = 574,
  kInfinite = 575,
  kRunning = 576,
  kPaused = 577,
  kFlat = 578,
  kPreserve3d = 579,
  kFillBox = 580,
  kViewBox = 581,
  kEase = 582,
  kLinear = 583,
  kEaseIn = 584,
  kEaseOut = 585,
  kEaseInOut = 586,
  kJumpBoth = 587,
  kJumpEnd = 588,
  kJumpNone = 589,
  kJumpStart = 590,
  kStepStart = 591,
  kStepEnd = 592,
  kSteps = 593,
  kFrames = 594,
  kCubicBezier = 595,
  kDocument = 596,
  kReset = 597,
  kZoom = 598,
  kVisiblepainted = 599,
  kVisiblefill = 600,
  kVisiblestroke = 601,
  kPainted = 602,
  kFill = 603,
  kStroke = 604,
  kBoundingBox = 605,
  kSpellOut = 606,
  kDigits = 607,
  kLiteralPunctuation = 608,
  kNoPunctuation = 609,
  kAntialiased = 610,
  kSubpixelAntialiased = 611,
  kOptimizespeed = 612,
  kOptimizelegibility = 613,
  kGeometricprecision = 614,
  kCrispedges = 615,
  kEconomy = 616,
  kExact = 617,
  kLr = 618,
  kRl = 619,
  kTb = 620,
  kLrTb = 621,
  kRlTb = 622,
  kTbRl = 623,
  kHorizontalTb = 624,
  kVerticalRl = 625,
  kVerticalLr = 626,
  kAfter = 627,
  kBefore = 628,
  kOver = 629,
  kUnder = 630,
  kFilled = 631,
  kOpen = 632,
  kDot = 633,
  kDoubleCircle = 634,
  kTriangle = 635,
  kSesame = 636,
  kEllipse = 637,
  kClosestSide = 638,
  kClosestCorner = 639,
  kFarthestSide = 640,
  kFarthestCorner = 641,
  kMixed = 642,
  kSideways = 643,
  kSidewaysRight = 644,
  kUpright = 645,
  kVerticalRight = 646,
  kOn = 647,
  kOff = 648,
  kOptimizequality = 649,
  kPixelated = 650,
  kWebkitOptimizeContrast = 651,
  kFromImage = 652,
  kRotateLeft = 653,
  kRotateRight = 654,
  kNonzero = 655,
  kEvenodd = 656,
  kAt = 657,
  kAlphabetic = 658,
  kBorderless = 659,
  kFullscreen = 660,
  kStandalone = 661,
  kMinimalUi = 662,
  kBrowser = 663,
  kWindowControlsOverlay = 664,
  kTabbed = 665,
  kPaged = 666,
  kSlow = 667,
  kFast = 668,
  kSticky = 669,
  kCoarse = 670,
  kFine = 671,
  kOnDemand = 672,
  kHover = 673,
  kMultiply = 674,
  kScreen = 675,
  kDarken = 676,
  kLighten = 677,
  kColorDodge = 678,
  kColorBurn = 679,
  kHardLight = 680,
  kSoftLight = 681,
  kDifference = 682,
  kExclusion = 683,
  kHue = 684,
  kSaturation = 685,
  kColor = 686,
  kLuminosity = 687,
  kScaleDown = 688,
  kBalance = 689,
  kDrag = 690,
  kNoDrag = 691,
  kSpan = 692,
  kMinmax = 693,
  kSubgrid = 694,
  kProgressive = 695,
  kInterlace = 696,
  kMarkers = 697,
  kAlpha = 698,
  kLuminance = 699,
  kSRGB = 700,
  kLinearrgb = 701,
  kButt = 702,
  kMiter = 703,
  kBevel = 704,
  kBeforeEdge = 705,
  kAfterEdge = 706,
  kCentral = 707,
  kTextBeforeEdge = 708,
  kTextAfterEdge = 709,
  kIdeographic = 710,
  kHanging = 711,
  kMathematical = 712,
  kUseScript = 713,
  kNoChange = 714,
  kResetSize = 715,
  kDynamic = 716,
  kNonScalingStroke = 717,
  kInternalExtendToZoom = 718,
  kPanX = 719,
  kPanY = 720,
  kPanLeft = 721,
  kPanRight = 722,
  kPanUp = 723,
  kPanDown = 724,
  kManipulation = 725,
  kPinchZoom = 726,
  kLastBaseline = 727,
  kFirstBaseline = 728,
  kFirst = 729,
  kLast = 730,
  kSelfStart = 731,
  kSelfEnd = 732,
  kLegacy = 733,
  kSmooth = 734,
  kScrollPosition = 735,
  kRevert = 736,
  kRevertLayer = 737,
  kUnset = 738,
  kLinearGradient = 739,
  kRadialGradient = 740,
  kConicGradient = 741,
  kRepeatingLinearGradient = 742,
  kRepeatingRadialGradient = 743,
  kRepeatingConicGradient = 744,
  kPaint = 745,
  kWebkitCrossFade = 746,
  kWebkitGradient = 747,
  kWebkitLinearGradient = 748,
  kWebkitRadialGradient = 749,
  kWebkitRepeatingLinearGradient = 750,
  kWebkitRepeatingRadialGradient = 751,
  kWebkitImageSet = 752,
  kImageSet = 753,
  kType = 754,
  kTo = 755,
  kColorStop = 756,
  kRadial = 757,
  kAttr = 758,
  kCounter = 759,
  kCounters = 760,
  kRect = 761,
  kPolygon = 762,
  kFormat = 763,
  kCollection = 764,
  kEmbeddedOpentype = 765,
  kOpentype = 766,
  kSVG = 767,
  kTruetype = 768,
  kWoff = 769,
  kWoff2 = 770,
  kTech = 771,
  kFeaturesOpentype = 772,
  kFeaturesAat = 773,
  kFeaturesGraphite = 774,
  kColorCOLRv0 = 775,
  kColorCOLRv1 = 776,
  kColorSVG = 777,
  kColorSbix = 778,
  kColorCBDT = 779,
  kVariations = 780,
  kPalettes = 781,
  kIncremental = 782,
  kInvert = 783,
  kGrayscale = 784,
  kSepia = 785,
  kSaturate = 786,
  kHueRotate = 787,
  kOpacity = 788,
  kBrightness = 789,
  kContrast = 790,
  kBlur = 791,
  kDropShadow = 792,
  kUrl = 793,
  kRgb = 794,
  kRgba = 795,
  kHsl = 796,
  kHsla = 797,
  kHwb = 798,
  kLab = 799,
  kOklab = 800,
  kLch = 801,
  kOklch = 802,
  kInternalLightDark = 803,
  kSRGBLinear = 804,
  kDisplayP3 = 805,
  kA98Rgb = 806,
  kProphotoRgb = 807,
  kXyz = 808,
  kXyzD50 = 809,
  kXyzD65 = 810,
  kShorter = 811,
  kLonger = 812,
  kDecreasing = 813,
  kIncreasing = 814,
  kIn = 815,
  kColorMix = 816,
  kFrom = 817,
  kR = 818,
  kG = 819,
  kB = 820,
  kH = 821,
  kS = 822,
  kL = 823,
  kW = 824,
  kA = 825,
  kC = 826,
  kMatrix = 827,
  kMatrix3d = 828,
  kPerspective = 829,
  kRotate = 830,
  kRotateX = 831,
  kRotateY = 832,
  kRotateZ = 833,
  kRotate3d = 834,
  kScale = 835,
  kScaleX = 836,
  kScaleY = 837,
  kScaleZ = 838,
  kScale3d = 839,
  kSkew = 840,
  kSkewX = 841,
  kSkewY = 842,
  kTranslate = 843,
  kTranslateX = 844,
  kTranslateY = 845,
  kTranslateZ = 846,
  kTranslate3d = 847,
  kX = 848,
  kY = 849,
  kZ = 850,
  kPath = 851,
  kRay = 852,
  kSides = 853,
  kStrokeBox = 854,
  kCalc = 855,
  kWebkitCalc = 856,
  kMin = 857,
  kMax = 858,
  kClamp = 859,
  kSin = 860,
  kCos = 861,
  kAsin = 862,
  kAtan = 863,
  kAtan2 = 864,
  kAcos = 865,
  kMod = 866,
  kRem = 867,
  kUp = 868,
  kDown = 869,
  kToZero = 870,
  kSign = 871,
  kAbs = 872,
  kPow = 873,
  kSqrt = 874,
  kHypot = 875,
  kLog = 876,
  kExp = 877,
  kInfinity = 878,
  kNegativeInfinity = 879,
  kNan = 880,
  kPi = 881,
  kE = 882,
  kMandatory = 883,
  kProximity = 884,
  kStyle = 885,
  kSize = 886,
  kBlockSize = 887,
  kInlineSize = 888,
  kState = 889,
  kInsetBlockStart = 890,
  kInsetBlockEnd = 891,
  kInsetInlineStart = 892,
  kInsetInlineEnd = 893,
  kAutoFill = 894,
  kAutoFit = 895,
  kVar = 896,
  kInternalVariableValue = 897,
  kEnv = 898,
  kAvoidPage = 899,
  kPage = 900,
  kRecto = 901,
  kVerso = 902,
  kAvoidColumn = 903,
  kP3 = 904,
  kRec2020 = 905,
  kAdd = 906,
  kAutoAdd = 907,
  kTrue = 908,
  kFalse = 909,
  kNoPreference = 910,
  kDark = 911,
  kLight = 912,
  kOnly = 913,
  kReduce = 914,
  kActive = 915,
  kPreserveParentColor = 916,
  kBackButton = 917,
  kFabricated = 918,
  kSelector = 919,
  kContinuous = 920,
  kFolded = 921,
  kStable = 922,
  kBothEdges = 923,
  kMore = 924,
  kLess = 925,
  kCustom = 926,
  kCyclic = 927,
  kSymbolic = 928,
  kNumeric = 929,
  kAdditive = 930,
  kExtends = 931,
  kInternalSimpChineseInformal = 932,
  kInternalSimpChineseFormal = 933,
  kInternalTradChineseInformal = 934,
  kInternalTradChineseFormal = 935,
  kInternalKoreanHangulFormal = 936,
  kInternalKoreanHanjaInformal = 937,
  kInternalKoreanHanjaFormal = 938,
  kInternalHebrew = 939,
  kInternalLowerArmenian = 940,
  kInternalUpperArmenian = 941,
  kInternalEthiopicNumeric = 942,
  kBullets = 943,
  kNumbers = 944,
  kWords = 945,
  kStandard = 946,
  kHigh = 947,
  kConstrainedHigh = 948,
  kLayer = 949,
  kGroup = 950,
  kSelf = 951,
  kCycle = 952,
  kCycleOn = 953,
  kPrev = 954,
  kNext = 955,
  kSet = 956,
  kToggle = 957,
  kColorContrast = 958,
  kVs = 959,
  kAA = 960,
  kAALarge = 961,
  kAAA = 962,
  kAAALarge = 963,
  kDrop = 964,
  kRaise = 965,
  kXywh = 966,
  kAnchor = 967,
  kAutoSame = 968,
  kAnchorSize = 969,
  kWidth = 970,
  kHeight = 971,
  kSelfBlock = 972,
  kSelfInline = 973,
  kImplicit = 974,
  kEntry = 975,
  kExit = 976,
  kEntryCrossing = 977,
  kExitCrossing = 978,
  kRoot = 979,
  kNearest = 980,
  kAllowDiscrete = 981,
  kInverted = 982,
  kCrossDocumentSameOrigin = 983,
  kSnap = 984,
  kEnabled = 985,
  kInitialOnly = 986,
};

const int numCSSValueKeywords = 987;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

CORE_EXPORT const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
