// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/runtime_feature_state_override_context.cc.tmpl
// and input files:
//   ../../third_party/blink/renderer/platform/runtime_enabled_features.json5


#include "third_party/blink/public/common/features.h"
#include "third_party/blink/public/common/origin_trials/origin_trial_feature.h"
#include "third_party/blink/public/common/thread_safe_browser_interface_broker_proxy.h"
#include "third_party/blink/public/mojom/runtime_feature_state/runtime_feature_state_controller.mojom-blink.h"
#include "third_party/blink/public/mojom/runtime_feature_state/runtime_feature_state.mojom.h"
#include "third_party/blink/public/platform/platform.h"
#include "third_party/blink/renderer/core/frame/web_feature.h"
#include "third_party/blink/renderer/platform/wtf/text/wtf_string.h"
#include "third_party/blink/renderer/platform/wtf/vector.h"
#include "third_party/blink/renderer/platform/runtime_feature_state/runtime_feature_state_override_context.h"

namespace blink {

void RuntimeFeatureStateOverrideContext::ApplyOverrideValuesFromParams(
    const base::flat_map<mojom::RuntimeFeatureState, bool>&
        values_from_params) {
  for(const auto& override_value : values_from_params) {
    override_values_[override_value.first] = override_value.second;
  }
}

bool RuntimeFeatureStateOverrideContext::
    IsDisableThirdPartyStoragePartitioningForceDisabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning);
  if (it != override_values_.end() && it->second == false)
    return true;

  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsDisableThirdPartyStoragePartitioningForceEnabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning);
  if(it != override_values_.end() && it->second == true)
    return true;

  return false;
}

void RuntimeFeatureStateOverrideContext::
    SetDisableThirdPartyStoragePartitioningForceDisabled() {
  override_values_
      [mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning] =
          false;
}

void RuntimeFeatureStateOverrideContext::
    SetDisableThirdPartyStoragePartitioningForceEnabled() {
  override_values_
      [mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning] =
          true;
}

bool RuntimeFeatureStateOverrideContext::
    SetDisableThirdPartyStoragePartitioningEnabled(const WTF::Vector<WTF::String>& tokens) {
  UseCounter::Count(use_counter_,
                          WebFeature::kDisableThirdPartyStoragePartitioning);
  // If the base::Feature for the deprecation trial is disabled, we should
  // not send any overrides to the browser, so we return false immediately,
  // skipping any mojom messaging.
  if(!base::FeatureList::IsEnabled(
    blink::features::kDisableThirdPartyStoragePartitioningDeprecationTrial)){
    return false;
  }
  // Origin Trial feature values should not override any sent from
  // the browser, so we only add a value if has not already been set
  // in the override_values_ map. Return whether the override was
  // successful.
  if(!override_values_.contains(
      mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning)) {
    // Update the state of all feature overrides in this class.
    override_values_
      [mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning] = true;
    // Add successful overrides to the list of features we sent back
    // to the browser process.
    mojom::blink::FeatureValuePtr value =
          mojom::blink::FeatureValue::New(true, tokens);
    origin_trial_overrides_[
      mojom::RuntimeFeatureState::kDisableThirdPartyStoragePartitioning] =
      std::move(value);
    return true;
  }
  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsFedCmIdpSigninStatusForceDisabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kFedCmIdpSigninStatus);
  if (it != override_values_.end() && it->second == false)
    return true;

  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsFedCmIdpSigninStatusForceEnabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kFedCmIdpSigninStatus);
  if(it != override_values_.end() && it->second == true)
    return true;

  return false;
}

void RuntimeFeatureStateOverrideContext::
    SetFedCmIdpSigninStatusForceDisabled() {
  override_values_
      [mojom::RuntimeFeatureState::kFedCmIdpSigninStatus] =
          false;
}

void RuntimeFeatureStateOverrideContext::
    SetFedCmIdpSigninStatusForceEnabled() {
  override_values_
      [mojom::RuntimeFeatureState::kFedCmIdpSigninStatus] =
          true;
}

bool RuntimeFeatureStateOverrideContext::
    SetFedCmIdpSigninStatusEnabled(const WTF::Vector<WTF::String>& tokens) {
  // Origin Trial feature values should not override any sent from
  // the browser, so we only add a value if has not already been set
  // in the override_values_ map. Return whether the override was
  // successful.
  if(!override_values_.contains(
      mojom::RuntimeFeatureState::kFedCmIdpSigninStatus)) {
    // Update the state of all feature overrides in this class.
    override_values_
      [mojom::RuntimeFeatureState::kFedCmIdpSigninStatus] = true;
    // Add successful overrides to the list of features we sent back
    // to the browser process.
    mojom::blink::FeatureValuePtr value =
          mojom::blink::FeatureValue::New(true, tokens);
    origin_trial_overrides_[
      mojom::RuntimeFeatureState::kFedCmIdpSigninStatus] =
      std::move(value);
    return true;
  }
  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsOriginTrialsSampleAPIBrowserReadWriteForceDisabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite);
  if (it != override_values_.end() && it->second == false)
    return true;

  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsOriginTrialsSampleAPIBrowserReadWriteForceEnabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite);
  if(it != override_values_.end() && it->second == true)
    return true;

  return false;
}

void RuntimeFeatureStateOverrideContext::
    SetOriginTrialsSampleAPIBrowserReadWriteForceDisabled() {
  override_values_
      [mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite] =
          false;
}

void RuntimeFeatureStateOverrideContext::
    SetOriginTrialsSampleAPIBrowserReadWriteForceEnabled() {
  override_values_
      [mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite] =
          true;
}

bool RuntimeFeatureStateOverrideContext::
    SetOriginTrialsSampleAPIBrowserReadWriteEnabled(const WTF::Vector<WTF::String>& tokens) {
  // Origin Trial feature values should not override any sent from
  // the browser, so we only add a value if has not already been set
  // in the override_values_ map. Return whether the override was
  // successful.
  if(!override_values_.contains(
      mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite)) {
    // Update the state of all feature overrides in this class.
    override_values_
      [mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite] = true;
    // Add successful overrides to the list of features we sent back
    // to the browser process.
    mojom::blink::FeatureValuePtr value =
          mojom::blink::FeatureValue::New(true, tokens);
    origin_trial_overrides_[
      mojom::RuntimeFeatureState::kOriginTrialsSampleAPIBrowserReadWrite] =
      std::move(value);
    return true;
  }
  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsTestFeatureForceDisabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kTestFeature);
  if (it != override_values_.end() && it->second == false)
    return true;

  return false;
}

bool RuntimeFeatureStateOverrideContext::
    IsTestFeatureForceEnabled() const {
  auto it = override_values_.find(
      mojom::RuntimeFeatureState::kTestFeature);
  if(it != override_values_.end() && it->second == true)
    return true;

  return false;
}

void RuntimeFeatureStateOverrideContext::
    SetTestFeatureForceDisabled() {
  override_values_
      [mojom::RuntimeFeatureState::kTestFeature] =
          false;
}

void RuntimeFeatureStateOverrideContext::
    SetTestFeatureForceEnabled() {
  override_values_
      [mojom::RuntimeFeatureState::kTestFeature] =
          true;
}

bool RuntimeFeatureStateOverrideContext::
    SetTestFeatureEnabled(const WTF::Vector<WTF::String>& tokens) {
  // Origin Trial feature values should not override any sent from
  // the browser, so we only add a value if has not already been set
  // in the override_values_ map. Return whether the override was
  // successful.
  if(!override_values_.contains(
      mojom::RuntimeFeatureState::kTestFeature)) {
    // Update the state of all feature overrides in this class.
    override_values_
      [mojom::RuntimeFeatureState::kTestFeature] = true;
    // Add successful overrides to the list of features we sent back
    // to the browser process.
    mojom::blink::FeatureValuePtr value =
          mojom::blink::FeatureValue::New(true, tokens);
    origin_trial_overrides_[
      mojom::RuntimeFeatureState::kTestFeature] =
      std::move(value);
    return true;
  }
  return false;
}


void RuntimeFeatureStateOverrideContext::ApplyOriginTrialOverride(
    const blink::OriginTrialFeature& origin_trial_feature,
    const WTF::Vector<WTF::String>& tokens) {
  // Determine which of the overrideable features has been set
  // via Origin Trial.
  bool overridden = false;
  if(origin_trial_feature == blink::OriginTrialFeature::kDisableThirdPartyStoragePartitioning) {
    overridden = SetDisableThirdPartyStoragePartitioningEnabled(tokens);
  }
  if(origin_trial_feature == blink::OriginTrialFeature::kFedCmIdpSigninStatus) {
    overridden = SetFedCmIdpSigninStatusEnabled(tokens);
  }
  if(origin_trial_feature == blink::OriginTrialFeature::kOriginTrialsSampleAPIBrowserReadWrite) {
    overridden = SetOriginTrialsSampleAPIBrowserReadWriteEnabled(tokens);
  }
  // Send the newly overriden values to the browser via IPC.
  if(overridden){
    // We must convert from std:: types to WTF:: types.
    // TODO(https://crbug.com/1377000): make origin_trial_overrides_ a
    // WTF:: type in a follow-up CL.
    WTF::HashMap<mojom::blink::RuntimeFeatureState,
                     mojom::blink::FeatureValuePtr> blink_overrides;
    for (auto& o : origin_trial_overrides_){
      blink_overrides.insert(o.first, std::move(o.second));
    }
    // Feature overrides and origin trial information should only be
    // sent to the browser process if the corresponding base::Feature
    // is enabled.
    if(base::FeatureList::IsEnabled(
        blink::features::kRuntimeFeatureStateControllerApplyFeatureDiff)){
      GetOrBindRuntimeFeatureStateController()->ApplyFeatureDiffForOriginTrial(
      std::move(blink_overrides));
    }
    // TODO(https://crbug.com/1377000): when we convert to WTF:: types,
    // we use std::move, so we have to do some cleanup.
    origin_trial_overrides_.clear();
  }
}

void RuntimeFeatureStateOverrideContext::EnablePersistentTrial(
      const WTF::String& raw_token,
      const WTF::Vector<scoped_refptr<const blink::SecurityOrigin>>&
          script_origins) {
    GetOrBindRuntimeFeatureStateController()->EnablePersistentTrial(
        raw_token, script_origins);
  }

mojom::blink::RuntimeFeatureStateController*
  RuntimeFeatureStateOverrideContext::GetOrBindRuntimeFeatureStateController(){
  // TODO(https://crbug.com/1410817): add support for workers/non-frames that
  // are enabling origin trials to send their information to the browser too.
  if (!runtime_feature_state_controller_remote_.is_bound()) {
      CHECK(send_runtime_features_to_browser_);
      binding_context_->GetBrowserInterfaceBroker().GetInterface(
          runtime_feature_state_controller_remote_.BindNewPipeAndPassReceiver(
              binding_context_->GetTaskRunner(TaskType::kInternalDefault)));
    }
  return runtime_feature_state_controller_remote_.get();
}

void RuntimeFeatureStateOverrideContext::Trace(Visitor* visitor) const{
  visitor->Trace(binding_context_);
  visitor->Trace(use_counter_);
  visitor->Trace(runtime_feature_state_controller_remote_);
}

}  // namespace blink
